import {Component, ElementRef, ViewChild} from '@angular/core';
import {FormControl, NgControl} from '@angular/forms';
import {Observable} from 'rxjs';
import {ISubscription} from 'rxjs/Subscription';
import {IAutoCompleteRow} from './autocomplete-row.interface';
import {AutoCompleteMapFunction} from './autocomplete-map-function.type';

@Component({
  template: `<div #wrapper class="input-autocomplete">
  <input #search type="text" [(ngModel)]="searchModel">
  <div class="input-autocomplete-current" [hidden]="!current">
    <span>{{getCurrentText()}}</span><i class="fi-x" (click)="reset()"></i>
  </div>
  <div class="input-autocomplete-loader" [hidden]="!loading"></div>
  <div #drop class="input-autocomplete-dropdown" [hidden]="!opened">
    <ul class="input-autocomplete-list" [hidden]="!hasData()">
      <li *ngFor="let row of data" (click)="select(row);"
          [ngClass]="row.classes">{{row.content}}
      </li>
    </ul>
    <p [hidden]="!isShowNoResult()">{{strings.noResult}}</p>
    <p [hidden]="!isShowInfo()">{{strings.help}}</p>
  </div>
</div>`
})
export class AutoCompleteComponent {
  public source: Function;
  public map: AutoCompleteMapFunction;
  public model: NgControl;

  public data: IAutoCompleteRow[];
  public current: IAutoCompleteRow;

  public opened: boolean = false;
  public loading: boolean = false;
  public minLength: number;
  public strings: {
    [key: string]: string
  };

  public searchModel: string = '';
  public sourceSubscription: ISubscription;

  @ViewChild('wrapper') wrapperRef: ElementRef;
  @ViewChild('search') searchRef: ElementRef;
  @ViewChild('drop') dropRef: ElementRef;


  ngAfterViewInit() {
    Observable.fromEvent(this.searchRef.nativeElement, 'focus')
      .debounceTime(200)
      .subscribe(() => {

        console.log('focus');
        this.open()
      });

    Observable.merge(
      Observable.fromEvent(this.searchRef.nativeElement, 'blur'),
      Observable.fromEvent(this.dropRef.nativeElement, 'click')
    )
      .debounceTime(210)
      .subscribe((e: any) => {
        console.log('blur');
        if (e instanceof FocusEvent) {
          this.cancel();
        }
      });

    const inputObs = Observable.merge(
      Observable.fromEvent(this.searchRef.nativeElement, 'focus'),
      Observable.fromEvent(this.searchRef.nativeElement, 'input'),
      Observable.fromEvent(this.searchRef.nativeElement, 'click')
    )
      .map((e: any) => {
        console.log('input');
        return e.target.value;
      })
      .filter((value: string) => {
        this.data = [];
        const valid = value.length >= this.minLength;
        if (valid) {
          this.loading = true;
        }
        return valid;
      })
      .debounceTime(200)
      .subscribe((search: string) => this.search(search));
  }

  getCurrentText() {
    return this.current ? this.current.label || this.current.content : '';
  }

  hasData(): boolean {
    return !!(this.data && this.data.length);
  }

  isShowNoResult(): boolean {
    return !this.loading && !this.hasData() && !this.isShowInfo() && !this.current;
  }

  isShowInfo(): boolean {
    let minLengthValid = (!this.searchModel || this.searchModel.length < this.minLength);
    return !this.loading && !this.hasData() && minLengthValid && !this.current && this.minLength !== 0;
  }

  setData(data: IAutoCompleteRow[]) {
    this.loading = false;
    this.data = data;
  }

  reset() {
    (<FormControl>this.model.control).setValue(null);
    this.current = null;
    this.searchModel = '';
    this.close();
  }

  private search(search: string) {
    if (this.sourceSubscription) {
      this.sourceSubscription.unsubscribe();
    }

    this.sourceSubscription = this.source(search).subscribe((data: any) => {
      this.setData(this.map(data));
      this.open();
    });
  }

  private select(row: IAutoCompleteRow) {
    if (row.selectable !== false) {
      this.cancel();
      if (row) {
        (<FormControl>this.model.control).setValue(row.id);
        this.current = row;
      }
    }
    return false;
  }

  private open() {
    if (!this.current) {
      this.opened = true;
    }
  }

  private close(clear?: boolean) {
    this.opened = false;
    if (clear) {
      this.setData([]);
    }
  }

  private cancel() {
    if (this.sourceSubscription) {
      this.sourceSubscription.unsubscribe();
    }
    this.close(true);
    this.searchModel = '';
  }
}
